"""Class: LazarusAuth

LazarusAuth holds and authenticates a user's Lazarus login credentials.
The Forms and RikAI classes must be initialized with a valid LazarusAuth
instance.

If the auth information provided is not valid, an InvalidAuthError
will be raised on initialization.
"""

import os
import sys
import requests

parent_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
sys.path.append(parent_dir)

from errors import InvalidAuthError

BASE_URL = os.environ.get("BASE_URL", "https://api.lazarusforms.com/")


class LazarusAuth:
    """A class to validate and store Lazarus auth credentials."""

    def __init__(self, org_id: str, auth_key: str):
        """Initialize a LazarusAuth() object.

        Org ID and Auth Key are authenticated on initialization.
        Initializing with invalid credentials will raise an error.

        Args:
            org_id (str): Lazarus organization ID
            auth_key (str): Lazarus authentication key
        """
        if not org_id or not auth_key:
            raise ValueError("Cannot initialize with an empty string.")
        self.headers = {"orgId": org_id, "authKey": auth_key}
        self.authenticate()


    def authenticate(self):
        """Authenticates Org ID and Auth Key.

        Posts a request to an API endpoint with no body. If the response
        is an AUTH_FAILURE with status code 403, we raise an error. Users
        will not be charged for this call as no pages are processed.

        Raises:
            InvalidAuthError
        """
        res = requests.post(f"{BASE_URL}/api/forms/generic", headers=self.headers)

        if res.status_code == 403:
            raise InvalidAuthError("Invalid org ID or auth key. Authentication failed.")


# LazarusAuth class usage examples
if __name__ == "__main__":
    org_id = os.environ.get("LAZARUS_ORG_ID")
    auth_key = os.environ.get("LAZARUS_AUTH_KEY")
    auth = LazarusAuth(org_id, auth_key)



""" Record library metrics """
import os
import requests


BASE_URL = os.environ.get("BASE_URL", "https://api.lazarusforms.com/")


def _record_metrics(endpoint: str, headers, model_id=None, response=None):
    """ Record metrics on successful and failed API requests using forms-python

    Args:
        endpoint (str): String indicating the endpoint in use
                Should be either "rikai", "forms", or "rikai/summarizer"
        headers: Authenticated header
        model_id (optional): Custom model ID. Defaults to None.
        response (optional): API response of successful requests. Defaults to None.
    """
    # Do not want to record metrics when running tests
    if os.getenv('TEST_MODE') == 'True':
        return

    metrics_url = f"{BASE_URL}/api/library-metrics/forms-python"
    if model_id:
        metrics_url += f"/{model_id}"

    data = {"endpoint": endpoint, "response": response}
    requests.post(metrics_url, headers=headers, json=data)



""" Lazarus Forms API Errors

Reformats API responses from failed requests into useful
error messages.
"""


class APIError(Exception):
    """Base class for errors returned from the API

    Attributes:
        status (str): Status of the API request
        message (str): Error message reported in the API request
        code (int): Response code from the API Request
        api_output (dict): Entire response from the API
    """

    def __init__(self, status, message, code, api_output=None):
        self.status = status
        self.message = message
        self.code = code
        self.api_output = api_output

    def __str__(self):
        """ Prints helpful message for user after traceback """
        error_string = ""
        if self.status is not None:
            error_string += f"\n\nSTATUS: {self.status}"
        if self.message is not None:
            error_string += f"\nMESSAGE: {self.message}"
        if self.code is not None:
            error_string += f"\nERROR CODE: {self.code}"
        if self.api_output is not None:
            error_string += f"\nAPI OUTPUT: {self.api_output}"

        return error_string


class AuthError(APIError):
    """Raised when API encounters an Authentication Failure """



from setuptools import setup, find_packages
setup(
  name = 'lazarus-ai',
  packages = ['lazarus_ai', 'errors', 'utils'],
  version = '1.0.0',
  license='MIT',
  description = 'Lazarus AI Python Library',
  author = 'Caroline Abel, KK Barrows, Alvin Xie',
  author_email = 'caroline@lazarus.enterprises, kathleen@lazarus.enterprises, alvin@lazarus.enterprises',
  url = 'https://www.lazarusforms.com/',
  download_url = 'https://github.com/Lazarus-AI/lazarus-ai-python/archive/refs/tags/v1.0.0.tar.gz',
  install_requires=[
          'requests',
          'stringcase',
      ],
  classifiers=[
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
  ],
)
